@@ -27,14 +27,17 @@ let lazy_env =
       Compat.init_path ();
       Ocaml_common.Compmisc.initial_env () )
 
-let string_of_lid lid =
-  let rec print lid acc =
-    match lid with
-    | Longident.Lident s -> s :: acc
-    | Longident.Ldot (lid, id) -> print lid ("." :: id :: acc)
-    | Longident.Lapply (la, lb) -> print la ("(" :: print lb (")" :: acc))
-  in
-  String.concat "" (print lid [])
+let string_of_lid' lid = Format.asprintf "%a" Pprintast.longident lid
+let string_of_lid lid = Format.asprintf "%a" Astlib.Pprintast.longident lid
+
+(* Remove locations *)
+let rec longident_without_loc =
+  let proj x = Compat.longident_proj x in
+  let conv x = longident_without_loc (proj x) in
+  function
+  | Longident.Ldot (x,y) -> Astlib.Longident.Ldot (conv x, proj y)
+  | Longident.Lapply (f,x) ->  Astlib.Longident.Lapply (conv f, conv x)
+  | Longident.Lident x -> Astlib.Longident.Lident x
 
 let try_find_module ~loc env lid =
   (* Note: we are careful to call `Env.lookup_module` and not
@@ -68,7 +71,7 @@ let try_find_module_type ~loc env lid =
       | None ->
         Location.raise_errorf ~loc
           "[%%import]: cannot access the signature of the abstract module %s"
-          (string_of_lid lid)
+          (string_of_lid' lid)
       | Some module_type -> module_type )
   with Not_found -> None
 
@@ -93,12 +96,16 @@ let open_module_type ~loc env lid module_type =
 let locate_sig ~loc env lid =
   let head, path =
     match Ppxlib.Longident.flatten_exn lid with
-    | head :: path -> (Longident.Lident head, path)
+    | head :: path -> (head, path)
     | _ -> assert false
+    | exception Invalid_argument _ ->
+      Location.raise_errorf ~loc
+        "[%%import] cannot import a functor application %s"
+          (string_of_lid lid)
   in
+  let chead = Longident.Lident head in
   let head_module_type =
-    match
-      (try_find_module ~loc env head, lazy (try_find_module_type ~loc env head))
+    match (try_find_module ~loc env chead, lazy (try_find_module_type ~loc env chead))
     with
     | Some mty, _ -> mty
     | None, (lazy (Some mty)) -> mty
@@ -124,10 +131,10 @@ let locate_sig ~loc env lid =
     let sub_module_type =
       loop (List.map Compat.migrate_signature_item sig_items)
     in
-    (Longident.Ldot (lid, path_item), sub_module_type)
+    (Astlib.Longident.Ldot(lid,path_item), sub_module_type)
   in
   let _lid, sub_module_type =
-    List.fold_left get_sub_module_type (head, head_module_type) path
+    List.fold_left get_sub_module_type (Astlib.Longident.Lident head, head_module_type) path
   in
   open_module_type ~loc env lid sub_module_type
 
@@ -164,14 +171,14 @@ let get_modtype_decl ~loc sig_items parent_lid elem =
       elem (string_of_lid parent_lid)
   | Some decl -> decl
 
-let longident_of_path = Untypeast.lident_of_path
+let longident_of_path x = longident_without_loc (Untypeast.lident_of_path x)
 
 let mknoloc (txt : 'a) : 'a Ppxlib.Location.loc =
   {txt; loc = Ppxlib.Location.none}
 
 let rec core_type_of_type_expr ~subst (type_expr : Ocaml_common.Types.type_expr)
     : Ppxlib.core_type =
-  match type_expr.desc with
+  match Compat.get_desc type_expr with
   | Tvar None -> Ppxlib.Ast_helper.Typ.any ()
   | Tvar (Some var) -> (
     match List.assoc (`Var var) subst with
@@ -187,8 +194,7 @@ let rec core_type_of_type_expr ~subst (type_expr : Ocaml_common.Types.type_expr)
       | _ -> lhs
     in
     Ppxlib.Ast_helper.Typ.arrow label lhs (core_type_of_type_expr ~subst rhs)
-  | Ttuple xs ->
-    Ppxlib.Ast_helper.Typ.tuple (List.map (core_type_of_type_expr ~subst) xs)
+  | Ttuple xs -> Compat.tuple (core_type_of_type_expr ~subst) xs
   | Tconstr (path, args, _) -> (
     let lid = longident_of_path path in
     let args = List.map (core_type_of_type_expr ~subst) args in
@@ -200,13 +206,13 @@ let rec core_type_of_type_expr ~subst (type_expr : Ocaml_common.Types.type_expr)
       Ppxlib.Ast_helper.Typ.constr
         {txt = longident_of_path path; loc = !Ppxlib.Ast_helper.default_loc}
         args )
-  | Tvariant {row_fields; _} ->
+  | Tvariant row_desc ->
     let fields =
-      row_fields
+      Compat.row_fields row_desc
       |> List.map (fun (label, row_field) ->
              let label = mknoloc label in
              let desc =
-               match row_field with
+               match Compat.row_field_repr row_field with
                | Types.Rpresent None -> Ppxlib.Rtag (label, true, [])
                | Types.Rpresent (Some ttyp) ->
                  Ppxlib.Rtag (label, false, [core_type_of_type_expr ~subst ttyp])
@@ -215,7 +221,7 @@ let rec core_type_of_type_expr ~subst (type_expr : Ocaml_common.Types.type_expr)
              Ppxlib.
                { prf_desc = desc
                ; prf_loc = !Ppxlib.Ast_helper.default_loc
-               ; prf_attributes = [] })
+               ; prf_attributes = [] } )
     in
     Ppxlib.Ast_helper.Typ.variant fields Closed None
   | _ -> assert false
@@ -232,10 +238,10 @@ let ptype_decl_of_ttype_decl ~manifest ~subst ptype_name
       try
         List.map2
           (fun (tparam : Ocaml_common.Types.type_expr) pparam ->
-            match tparam with
-            | {desc = Tvar (Some var); _} -> [(`Var var, pparam)]
-            | {desc = Tvar None; _} -> []
-            | _ -> assert false)
+            match Compat.get_desc tparam with
+            | Tvar (Some var) -> [(`Var var, pparam)]
+            | Tvar None -> []
+            | _ -> assert false )
           ttype_decl.type_params ptype_args
         |> List.concat
       with Invalid_argument _ ->
@@ -253,7 +259,7 @@ let ptype_decl_of_ttype_decl ~manifest ~subst ptype_name
         , (* The equivalent of not specifying the variance explicitly.
              Since the very purpose of ppx_import is to include the full definition,
              it should always be sufficient to rely on the inferencer to deduce variance. *)
-          (Ppxlib.Asttypes.NoVariance, Ppxlib.Asttypes.NoInjectivity) ))
+          (Ppxlib.Asttypes.NoVariance, Ppxlib.Asttypes.NoInjectivity) ) )
       ttype_decl.type_params ttype_decl.type_variance
   and ptype_kind =
     let map_labels =
@@ -264,7 +270,7 @@ let ptype_decl_of_ttype_decl ~manifest ~subst ptype_name
             ; pld_mutable = Tt.copy_mutable_flag ld.ld_mutable
             ; pld_type = core_type_of_type_expr ~subst ld.ld_type
             ; pld_loc = ld.ld_loc
-            ; pld_attributes = Tt.copy_attributes ld.ld_attributes })
+            ; pld_attributes = Tt.copy_attributes ld.ld_attributes } )
     in
     Ppxlib.(
       match Compat.migrate_type_kind ttype_decl.type_kind with
@@ -288,10 +294,11 @@ let ptype_decl_of_ttype_decl ~manifest ~subst ptype_name
                  in
                  { pcd_name =
                      {txt = Ocaml_common.Ident.name cd.cd_id; loc = cd.cd_loc}
+                 ; pcd_vars = []
                  ; pcd_args = map_args cd.cd_args
                  ; pcd_res
                  ; pcd_loc = cd.cd_loc
-                 ; pcd_attributes = Tt.copy_attributes cd.cd_attributes }) ))
+                 ; pcd_attributes = Tt.copy_attributes cd.cd_attributes } ) ))
   and ptype_manifest =
     match ttype_decl.type_manifest with
     | Some typ -> Some (core_type_of_type_expr ~subst typ)
@@ -361,8 +368,8 @@ let is_self_reference ~input_name lid =
     fn = mn
   | _ -> false
 
-let type_declaration ~tool_name ~input_name
-    (type_decl : Ppxlib.type_declaration) =
+let type_declaration ~tool_name ~input_name (type_decl : Ppxlib.type_declaration)
+    =
   let open Ppxlib in
   match type_decl with
   | { ptype_attributes
@@ -386,11 +393,11 @@ let type_declaration ~tool_name ~input_name
                 Location.raise_errorf ~loc
                   "[%%import] cannot import a functor application %s"
                   (string_of_lid lid)
-              | Lident _ as head_id ->
+              | Lident hid ->
                 (* In this case, we know for sure that the user intends this lident
                    as a type name, so we use Typetexp.find_type and let the failure
                    cases propagate to the user. *)
-                Compat.find_type env ~loc head_id |> snd
+                Compat.find_type env ~loc (Lident hid) |> snd
               | Ldot (parent_id, elem) ->
                 let sig_items = locate_sig ~loc env parent_id in
                 get_type_decl ~loc sig_items parent_id elem
@@ -412,7 +419,7 @@ let type_declaration ~tool_name ~input_name
               ptype_decl_of_ttype_decl ~manifest:m ~subst:s ptype_name
                 ttype_decl
             in
-            {ptype_decl with ptype_attributes})
+            {ptype_decl with ptype_attributes} )
     | _ -> Location.raise_errorf ~loc "Invalid [%%import] syntax" )
   | _ -> type_decl
 
@@ -440,7 +447,7 @@ let rec psig_of_tsig ~subst (tsig : Compat.signature_item_407 list) :
       |> List.map (fun (id, ttype_decl) ->
              ptype_decl_of_ttype_decl ~manifest:None ~subst
                (mknoloc (Ocaml_common.Ident.name id))
-               ttype_decl)
+               ttype_decl )
     in
     let psig_desc = Psig_type (rec_flag, block) in
     {psig_desc; psig_loc = Location.none} :: psig_of_tsig ~subst rest
@@ -465,16 +472,14 @@ let rec psig_of_tsig ~subst (tsig : Compat.signature_item_407 list) :
   | [] -> []
   | _ -> assert false
 
-let module_type ~tool_name ~input_name
-    (package_type : Ppxlib.package_type) =
+let module_type ~tool_name ~input_name (package_type : Ppxlib.package_type) =
   let open Ppxlib in
   let ({txt = lid; loc} as alias), subst = package_type in
   if tool_name = "ocamldep" then
     if is_self_reference ~input_name lid then
       (* Create a dummy module type to break the circular dependency *)
       Ast_helper.Mty.mk ~attrs:[] (Pmty_signature [])
-    else
-      (* Just put it as alias *)
+    else (* Just put it as alias *)
       Ast_helper.Mty.mk ~attrs:[] (Pmty_alias alias)
   else
     Ppxlib.Ast_helper.with_default_loc loc (fun () ->
@@ -485,11 +490,11 @@ let module_type ~tool_name ~input_name
             Location.raise_errorf ~loc
               "[%%import] cannot import a functor application %s"
               (string_of_lid lid)
-          | Longident.Lident _ as head_id ->
+          | Longident.Lident head_id ->
             (* In this case, we know for sure that the user intends this lident
                as a module type name, so we use Typetexp.find_type and
                let the failure cases propagate to the user. *)
-            Compat.find_modtype env ~loc head_id |> snd
+            Compat.find_modtype env ~loc (Lident head_id) |> snd
           | Longident.Ldot (parent_id, elem) ->
             let sig_items = locate_sig ~loc env parent_id in
             get_modtype_decl ~loc sig_items parent_id elem
@@ -504,7 +509,7 @@ let module_type ~tool_name ~input_name
         | {mtd_type = None; _} ->
           Location.raise_errorf ~loc "Imported module is abstract"
         | _ ->
-          Location.raise_errorf ~loc "Imported module is indirectly defined")
+          Location.raise_errorf ~loc "Imported module is indirectly defined" )
 
 let type_declaration_expand ~ctxt type_decl =
   let tool_name = Ppxlib.Expansion_context.Extension.tool_name ctxt in
